/******************************************************************************
 * Coach.hh
 * Copyright (c) 2003 Virtual Werder
 *
 *
 * implements:
 *   class Coach
 *
 * requires:
 *
 *
 * Created on: 05.05.2003 by pagra
 * Maintainer: pagra
 * Group:
 * Co-authors:
 *****************************************************************************/

#ifndef COACH_HH
#define COACH_HH

#include "common.hh"
#include "Parser.hh"
#include "ParsedMsg.hh"
#include "PlayerType.hh"

class Communicator;
class Parser;
class ParsedMsg;

class WorldModel;

/**
 This is the main Coach class.
 It communicates with the server (by the communicator) and distributes
 the heterogenous players to his team.
 @ingroup Main
 @brief class Coach in group Main
 */
class Coach {
public:
  /**
   * constructor
   */
  Coach( /*history, adviceQueue*/ );

  /**
   * destructor
   */
  ~Coach();

  /**
   * Constructor
   * @param *wm pointer to the worldmodel,defined in main.cc
   */
	Coach (WorldModel *wm);

  /**
   * initializes the coach
   * @param const char *serverHost name of the server to connect to.
   * @param int serverPort port on which the server runs
   * @param const char *teamname name of the team to coach
   */
  void init( const char *serverHost, int serverPort, const char *teamname );

  /**
   * starts the coach
   */
  void run();

private:
  /**
   * The parser object
   */
  Parser *parser;

  /**
   * The communicator, to communicate with the server
   */
  Communicator *communicator;

  /**
   * The worldmodel of the coach
   */
  WorldModel *worldmodel;

  const char *teamname;
  FieldSide ourSide;
  vector<PlayerType> playerTypes; // TODO (not always 7 player types)
  //int playerTypesSend[7]; // how often a player type has been sent/chosen
  int playmode;
  unsigned int serverTime;
  unsigned int differentPlayerTypes; // <-- remove me (already exists in pParam->player_types)
  unsigned int maximumSubstitutions; // <-- remove me (already exists in pParam->pt_max)
  bool isSynchronizedMode;
  ServerParamMsg *sParam;
  PlayerParamMsg *pParam;

  /**
   * This method sends the calculated values to the team
   */
  void sendDetectedPlayerTypes();
  void sendDetectedOffsideX();
  void sendFormation();
  void sendDetectedValues();
  void sendGraphicPart(int x, int y);
  void sendGraphic();

  /**
   * integrates a message from the server into the coach/worldmodel
   */
  void integrateMessage( ParsedMsg *parsedMsg );

  void testThink(); // test function (josh)
};


#endif // COACH_HH
