/******************************************************************************
 * ParsedMsg.cc
 * Copyright (c) 2003 Virtual Werder
 *
 *
 * implements:
 *   all messages received by the coach, that have at least one data parameter
 *   class ParsedMsg
 *   InitSideMsg
 *
 * requires:
 *
 *
 * Created on: 07.05.2003 by pagra
 * Maintainer: pagra
 * Group:
 * Co-authors:
 *****************************************************************************/

#include "ParsedMsg.hh"
#include "ParsedObjects.hh"
#include "common.hh"
#include <stdlib.h>
#include <iostream>

////////////////////

ParsedMsg::ParsedMsg( ParsedMsgType msgType ) {
  this->msgType = msgType;
}

ParsedMsg::~ParsedMsg() {}

////////////////////

InitSideOkMsg::InitSideOkMsg() : ParsedMsg( InitSideOk_Msg ) {
  fieldSide = FS_Unknown;
}

InitSideOkMsg::~InitSideOkMsg() {}

////////////////////

OkEyeMsg::OkEyeMsg() : ParsedMsg( OkEye_Msg ) {
  eyeOn = false;
}

OkEyeMsg::~OkEyeMsg() {}

////////////////////

OkChangePlayerTypeMsg::OkChangePlayerTypeMsg() : ParsedMsg( OkChangePlayerType_Msg ) {
  uNum = UNDEFINED_INT;
  playerType = UNDEFINED_INT;
}

OkChangePlayerTypeMsg::~OkChangePlayerTypeMsg() {}

ChangePlayerTypeMsg::ChangePlayerTypeMsg() : ParsedMsg ( ChangePlayerType_Msg ) {
  uNum = UNDEFINED_INT; 
}

ChangePlayerTypeMsg::~ChangePlayerTypeMsg() {}

WNoTeamFoundMsg::WNoTeamFoundMsg() : ParsedMsg( W_NoTeamFound ) {}
WNoTeamFoundMsg::~WNoTeamFoundMsg() {}

WNoSuchPlayerMsg::WNoSuchPlayerMsg() : ParsedMsg( W_NoSuchPlayer ) {}
WNoSuchPlayerMsg::~WNoSuchPlayerMsg() {}

WCannotSubWhilePlayOnMsg::WCannotSubWhilePlayOnMsg() : ParsedMsg( W_CannotSubWhilePlayOn ) {}
WCannotSubWhilePlayOnMsg::~WCannotSubWhilePlayOnMsg() {}

WNoSubsLeftMsg::WNoSubsLeftMsg() : ParsedMsg( W_NoSubsLeft ) {}
WNoSubsLeftMsg::~WNoSubsLeftMsg() {}

WMaxOfThatPlayerTypeOnFieldMsg::WMaxOfThatPlayerTypeOnFieldMsg() 
  : ParsedMsg( W_MaxOfThatPlayerTypeOnField) {}
WMaxOfThatPlayerTypeOnFieldMsg::~WMaxOfThatPlayerTypeOnFieldMsg() {}

WCannotChangeGoalieMsg::WCannotChangeGoalieMsg() : ParsedMsg( W_CannotChangeGoalie ) {}
WCannotChangeGoalieMsg::~WCannotChangeGoalieMsg() {}

////////////////////

OkTeamnamesMsg::OkTeamnamesMsg() : ParsedMsg( OkTeamnames_Msg ) {
  teamnameLeft  = NULL;
  teamnameRight = NULL;
}

OkTeamnamesMsg::~OkTeamnamesMsg() {
  if ( teamnameLeft != NULL ) {
    free((void *)teamnameLeft);
  }
  if ( teamnameRight != NULL ) {
    free((void *)teamnameRight);
  }
}

void OkTeamnamesMsg::print() const {
  cout << "OkTeamnamesMsg parsed";
  if ( teamnameLeft != NULL ) {
    cout << ", left team: " << teamnameLeft;
  }
  if ( teamnameRight != NULL ) {
    cout << ", right team: " << teamnameRight;
  }
  cout << endl;
}

////////////////////

OkSayMsg::OkSayMsg() : ParsedMsg( OkSay_Msg ) {}
OkSayMsg::~OkSayMsg() {}

////////////////////

ServerParamMsg::ServerParamMsg() : ParsedMsg( ServerParam_Msg ) {
  audio_cut_dist = UNDEFINED_DOUBLE;
  back_passes = UNDEFINED_INT;
  ball_accel_max = UNDEFINED_DOUBLE;
  ball_decay = UNDEFINED_DOUBLE;
  ball_rand = UNDEFINED_DOUBLE;
  ball_size = UNDEFINED_DOUBLE;
  ball_speed_max = UNDEFINED_DOUBLE;
  ball_weight = UNDEFINED_DOUBLE;
  catch_ban_cycle = UNDEFINED_INT;
  catch_probability = UNDEFINED_DOUBLE;
  catchable_area_l = UNDEFINED_DOUBLE;
  catchable_area_w = UNDEFINED_DOUBLE;
  ckick_margin = UNDEFINED_INT;
  clang_advice_win = UNDEFINED_INT;
  clang_define_win = UNDEFINED_INT;
  clang_del_win = UNDEFINED_INT;
  clang_info_win = UNDEFINED_INT;
  clang_mess_delay = UNDEFINED_INT;
  clang_mess_per_cycle = UNDEFINED_INT;
  clang_meta_win = UNDEFINED_INT;
  clang_rule_win = UNDEFINED_INT;
  clang_win_size = UNDEFINED_INT;
  dash_power_rate = UNDEFINED_DOUBLE;
  drop_ball_time = UNDEFINED_INT;
  effort_dec = UNDEFINED_DOUBLE;
  effort_dec_thr = UNDEFINED_DOUBLE;
  effort_inc = UNDEFINED_DOUBLE;
  effort_inc_thr = UNDEFINED_DOUBLE;
  effort_init = UNDEFINED_DOUBLE;
  effort_min = UNDEFINED_DOUBLE;
  forbid_kick_off_offside = UNDEFINED_INT;
  free_kick_faults = UNDEFINED_INT;
  freeform_send_period = UNDEFINED_INT;
  freeform_wait_period = UNDEFINED_INT;
  fullstate_l = UNDEFINED_INT;
  fullstate_r = UNDEFINED_INT;
  goal_width = UNDEFINED_DOUBLE;
  goalie_max_moves = UNDEFINED_INT;
  half_time = UNDEFINED_INT;
  hear_decay = UNDEFINED_INT;
  hear_inc = UNDEFINED_INT;
  hear_max = UNDEFINED_INT;
  inertia_moment = UNDEFINED_DOUBLE;
  kick_power_rate = UNDEFINED_DOUBLE;
  kick_rand = UNDEFINED_DOUBLE;
  kick_rand_factor_l = UNDEFINED_DOUBLE;
  kick_rand_factor_r = UNDEFINED_DOUBLE;
  kickable_margin = UNDEFINED_INT;
  max_goal_kicks = UNDEFINED_INT;
  maxmoment = UNDEFINED_DOUBLE;
  maxneckang = UNDEFINED_DOUBLE;
  maxneckmoment = UNDEFINED_DOUBLE;
  maxpower = UNDEFINED_DOUBLE;
  minmoment = UNDEFINED_DOUBLE;
  minneckang = UNDEFINED_DOUBLE;
  minneckmoment = UNDEFINED_DOUBLE;
  minpower = UNDEFINED_DOUBLE;
  offside_active_area_size = UNDEFINED_DOUBLE;
  offside_kick_margin = UNDEFINED_DOUBLE;
  player_accel_max = UNDEFINED_DOUBLE;
  player_decay = UNDEFINED_DOUBLE;
  player_rand = UNDEFINED_DOUBLE;
  player_size = UNDEFINED_DOUBLE;
  player_speed_max = UNDEFINED_DOUBLE;
  player_weight = UNDEFINED_DOUBLE;
  point_to_ban = UNDEFINED_INT;
  point_to_duration = UNDEFINED_INT;
  prand_factor_l = UNDEFINED_DOUBLE;
  prand_factor_r = UNDEFINED_DOUBLE;
  proper_goal_kicks = UNDEFINED_INT;
  quantize_step = UNDEFINED_DOUBLE;
  quantize_step_l = UNDEFINED_DOUBLE;
  recover_dec = UNDEFINED_DOUBLE;
  recover_dec_thr = UNDEFINED_DOUBLE;
  recover_min = UNDEFINED_DOUBLE;
  recv_step = UNDEFINED_DOUBLE;
  say_coach_cnt_max = UNDEFINED_INT;
  say_coach_msg_size = UNDEFINED_INT;
  say_msg_size = UNDEFINED_INT;
  send_comms = UNDEFINED_INT;
  send_step = UNDEFINED_INT;
  send_vi_step = UNDEFINED_INT;
  sense_body_step = UNDEFINED_INT;
  simulator_step = UNDEFINED_INT;
  slow_down_factor = UNDEFINED_DOUBLE;
  slowness_on_top_for_left_team = UNDEFINED_INT;
  slowness_on_top_for_right_team = UNDEFINED_INT;
  stamina_inc_max = UNDEFINED_DOUBLE;
  stamina_max = UNDEFINED_DOUBLE;
  start_goal_l = UNDEFINED_INT;
  start_goal_r = UNDEFINED_INT;
  stopped_ball_vel = UNDEFINED_DOUBLE;
  synch_micro_sleep = UNDEFINED_INT;
  synch_mode = UNDEFINED_INT;
  synch_offset = UNDEFINED_INT;
  tackle_back_dist = UNDEFINED_DOUBLE;
  tackle_cycles = UNDEFINED_INT;
  tackle_dist = UNDEFINED_DOUBLE;
  tackle_exponent = UNDEFINED_DOUBLE;
  tackle_power_rate = UNDEFINED_DOUBLE;
  tackle_width = UNDEFINED_DOUBLE;
  use_offside = UNDEFINED_INT;
  visible_angle = UNDEFINED_DOUBLE;
  visible_distance = UNDEFINED_DOUBLE;
  wind_ang = UNDEFINED_DOUBLE;
  wind_dir = UNDEFINED_DOUBLE;
  wind_force = UNDEFINED_DOUBLE;
  wind_none = UNDEFINED_INT;
  wind_rand = UNDEFINED_DOUBLE;
  wind_random = UNDEFINED_INT;
}

ServerParamMsg::~ServerParamMsg() {}

////////////////////

PlayerParamMsg::PlayerParamMsg() : ParsedMsg( PlayerParam_Msg ) {
  player_types = UNDEFINED_INT;
  pt_max = UNDEFINED_INT;
  random_seed = UNDEFINED_INT;
  subs_max = UNDEFINED_INT;
  
  dash_power_rate_delta_max = UNDEFINED_DOUBLE;
  dash_power_rate_delta_min = UNDEFINED_DOUBLE;
  effort_max_delta_factor = UNDEFINED_DOUBLE;
  effort_min_delta_factor = UNDEFINED_DOUBLE;
  extra_stamina_delta_max = UNDEFINED_DOUBLE;
  extra_stamina_delta_min = UNDEFINED_DOUBLE;
  inertia_moment_delta_factor = UNDEFINED_DOUBLE;
  kick_rand_delta_factor = UNDEFINED_DOUBLE;
  kickable_margin_delta_max = UNDEFINED_DOUBLE;
  kickable_margin_delta_min = UNDEFINED_DOUBLE;
  new_dash_power_rate_delta_max = UNDEFINED_DOUBLE;
  new_dash_power_rate_delta_min = UNDEFINED_DOUBLE;
  new_stamina_inc_max_delta_factor = UNDEFINED_DOUBLE;
  player_decay_delta_max = UNDEFINED_DOUBLE;
  player_decay_delta_min = UNDEFINED_DOUBLE;
  player_size_delta_factor = UNDEFINED_DOUBLE;
  player_speed_max_delta_max = UNDEFINED_DOUBLE;
  player_speed_max_delta_min = UNDEFINED_DOUBLE;
  stamina_inc_max_delta_factor = UNDEFINED_DOUBLE;
}

PlayerParamMsg::~PlayerParamMsg() {}

void PlayerParamMsg::print() const {
  cout << "PlayerParamMsg parsed: " << endl
       << "player_types: " << player_types
       << "  pt_max: " << pt_max
       << "  random_seed: " << random_seed
       << "  subs_max: " << subs_max
       << "  dash_power_rate_delta_max: " << dash_power_rate_delta_max
       << "  dash_power_rate_delta_min: " << dash_power_rate_delta_min
       << "  effort_max_delta_factor: " << effort_max_delta_factor
       << "  effort_min_delta_factor: " << effort_min_delta_factor
       << "  extra_stamina_delta_max: " << extra_stamina_delta_max
       << "  extra_stamina_delta_min: " << extra_stamina_delta_min
       << "  inertia_moment_delta_factor: " << inertia_moment_delta_factor
       << "  kick_rand_delta_factor: " << kick_rand_delta_factor
       << "  kickable_margin_delta_max: " << kickable_margin_delta_max
       << "  kickable_margin_delta_min: " << kickable_margin_delta_min
       << "  new_dash_power_rate_delta_max: " << new_dash_power_rate_delta_max
       << "  new_dash_power_rate_delta_min: " << new_dash_power_rate_delta_min
       << "  new_stamina_inc_max_delta_factor: " << new_stamina_inc_max_delta_factor
       << "  player_decay_delta_max: " << player_decay_delta_max
       << "  player_decay_delta_min: " << player_decay_delta_min
       << "  player_size_delta_factor: " << player_size_delta_factor
       << "  player_speed_max_delta_max: " << player_speed_max_delta_max
       << "  player_speed_max_delta_min: " << player_speed_max_delta_min
       << "  stamina_inc_max_delta_factor: " << stamina_inc_max_delta_factor << endl;
}

/////////////////

PlayerTypeMsg::PlayerTypeMsg() : ParsedMsg( PlayerType_Msg ) {
  id = UNDEFINED_INT;
  player_speed_max = UNDEFINED_DOUBLE;
  stamina_inc_max = UNDEFINED_DOUBLE;
  player_decay = UNDEFINED_DOUBLE;
  inertia_moment = UNDEFINED_DOUBLE;
  dash_power_rate = UNDEFINED_DOUBLE;
  player_size = UNDEFINED_DOUBLE;
  kickable_margin = UNDEFINED_DOUBLE;
  kick_rand = UNDEFINED_DOUBLE;
  extra_stamina = UNDEFINED_DOUBLE;
  effort_max = UNDEFINED_DOUBLE;
  effort_min = UNDEFINED_DOUBLE;
}

PlayerTypeMsg::~PlayerTypeMsg() {}

///////////////////

VisualMsg::VisualMsg( ParsedMsgType msgType ) : ParsedMsg( msgType ) {
  serverTime = UNDEFINED_INT;
  ball = new ParsedBall();
  ourTeam = new vector<ParsedPlayer *>();
  theirTeam = new vector<ParsedPlayer *>();
}

VisualMsg::~VisualMsg() {
  vector<ParsedPlayer *>::iterator i;
  for ( i = ourTeam->begin(); i != ourTeam->end(); ++i ) {
    delete (*i);
  }
  delete ourTeam;

  for ( i = theirTeam->begin(); i != theirTeam->end(); ++i ) {
    delete (*i);
  }
  delete theirTeam;

  delete ball;
}

void VisualMsg::print() const {
  cout << "VisualMsg parsed (Time=" << serverTime << ")" << endl;
  printBallAndTeams();
}

void VisualMsg::printBallAndTeams() const {
  cout << "Ball: " << ball->posX << " " << ball->posY << " " 
       << ball->velX << " " << ball->velY << endl;

  vector<ParsedPlayer *>::const_iterator i;

  for ( i = ourTeam->begin(); i != ourTeam->end(); ++i ) {
    cout << "our: uNum " << (*i)->uNum << ": " << (*i)->posX << " " << (*i)->posY << " "
	 << (*i)->velX << " " << (*i)->velY << " "
	 << (*i)->bodyAngle << " " << (*i)->neckAngle;
    if ( (*i)->isGoalie() ) {
      cout << " goalie";
    }
    cout << endl;
  }

  for ( i = theirTeam->begin(); i != theirTeam->end(); ++i ) {
    cout << "their: uNum " << (*i)->uNum << ": " << (*i)->posX << " " << (*i)->posY << " "
	 << (*i)->velX << " " << (*i)->velY << " "
	 << (*i)->bodyAngle << " " << (*i)->neckAngle;
    if ( (*i)->isGoalie() ) {
      cout << " goalie";
    }
    cout << endl;
  }
  cout << endl;
}

SeeGlobalMsg::SeeGlobalMsg() : VisualMsg( SeeGlobal_Msg ) {}

SeeGlobalMsg::~SeeGlobalMsg() {}

void SeeGlobalMsg::print() const {
  cout << "SeeGlobalMsg parsed (Time=" << serverTime << ")" << endl;
  printBallAndTeams();
}

OkLookMsg::OkLookMsg() : VisualMsg( OkLook_Msg ) {}

OkLookMsg::~OkLookMsg() {}

void OkLookMsg::print() const {
  cout << "OkLookMsg parsed (Time=" << serverTime << ")" << endl;
  printBallAndTeams();
}

////////////////////

HearRefereeMsg::HearRefereeMsg() : ParsedMsg( HearReferee_Msg ) {
  serverTime = UNDEFINED_INT;
  playmode = PM_UNKNOWN;
}

HearRefereeMsg::~HearRefereeMsg() {}

void HearRefereeMsg::print() const {
  cout << "HearRefereeMsg parsed (Time=" << serverTime << "): Playmode: " 
       << playmode << endl;
}

////////////////////

HearPlayerMsg::HearPlayerMsg() : ParsedMsg( HearPlayer_Msg ) {
  serverTime = UNDEFINED_INT;
  uNum = UNDEFINED_INT;
  ourTeam = false;
  message = NULL;
}

HearPlayerMsg::~HearPlayerMsg() {
  if ( message != NULL ) {
    free((void *)message);
  }
}

void HearPlayerMsg::print() const {
  cout << "HearPlayerMsg parsed: Time=" << serverTime << "  Team: ";
  if ( ourTeam ) cout << "our";
  else cout << "their";
  cout << "  uNum: " << uNum;
  if ( message != NULL ) cout << "  msg: " << message << endl;
  else cout << "  no message received!" << endl;
}

TeamGraphicOkMsg::TeamGraphicOkMsg() : ParsedMsg( TeamGraphicOk_Msg ){
  x = 0; y = 0;
}
TeamGraphicOkMsg::~TeamGraphicOkMsg(){
}

////////////////////

CLangVersPlayerMsg::CLangVersPlayerMsg() : ParsedMsg( CLangVersPlayer_Msg ) {
  uNum = UNDEFINED_INT;
  ourTeam = false;
  maxCLangVers = UNDEFINED_INT;
  minCLangVers = UNDEFINED_INT;
}

CLangVersPlayerMsg::~CLangVersPlayerMsg() {}

void CLangVersPlayerMsg::print() const {
  cout << "CLangVersPlayerMsg parsed: ";
  if ( ourTeam ) cout << "our Team";
  else cout << "their Team";
  cout << ", uNum: " << uNum << ", Min: " << minCLangVers 
       << ", Max: " << maxCLangVers << endl;
}

////////////////////

ENoMoreTeamOrPlayerOrGoalieMsg::ENoMoreTeamOrPlayerOrGoalieMsg() 
  : ParsedMsg( E_NoMoreTeamOrPlayerOrGoalie ) {}
ENoMoreTeamOrPlayerOrGoalieMsg::~ENoMoreTeamOrPlayerOrGoalieMsg() {}
ENoSuchTeamOrAlreadyHaveCoachMsg::ENoSuchTeamOrAlreadyHaveCoachMsg() 
  : ParsedMsg( E_NoSuchTeamOrAlreadyHaveCoach ) {}
ENoSuchTeamOrAlreadyHaveCoachMsg::~ENoSuchTeamOrAlreadyHaveCoachMsg() {}
EUnknownCommandMsg::EUnknownCommandMsg() : ParsedMsg( E_UnknownCommand ) {}
EUnknownCommandMsg::~EUnknownCommandMsg() {}
EIllegalCommandFormMsg::EIllegalCommandFormMsg() : ParsedMsg( E_IllegalCommandForm ) {}
EIllegalCommandFormMsg::~EIllegalCommandFormMsg() {}
EOutOfRangePlayerTypeMsg::EOutOfRangePlayerTypeMsg() : ParsedMsg( E_OutOfRangePlayerType ) {}
EOutOfRangePlayerTypeMsg::~EOutOfRangePlayerTypeMsg() {}
EIllegalModeMsg::EIllegalModeMsg() : ParsedMsg( E_IllegalMode ) {}
EIllegalModeMsg::~EIllegalModeMsg() {}
EIllegalObjectFormMsg::EIllegalObjectFormMsg() : ParsedMsg( E_IllegalObjectForm ) {}
EIllegalObjectFormMsg::~EIllegalObjectFormMsg() {}
ESaidTooManyFreeformMessagesMsg::ESaidTooManyFreeformMessagesMsg() 
  : ParsedMsg( E_SaidTooManyFreeformMessages ) {}
ESaidTooManyFreeformMessagesMsg::~ESaidTooManyFreeformMessagesMsg() {}
ECannotSayFreeformWhilePlayonMsg::ECannotSayFreeformWhilePlayonMsg() 
  : ParsedMsg( E_CannotSayFreeformWhilePlayon ) {}
ECannotSayFreeformWhilePlayonMsg::~ECannotSayFreeformWhilePlayonMsg() {}
ESaidTooManyMetaMessagesMsg::ESaidTooManyMetaMessagesMsg() 
  : ParsedMsg( E_SaidTooManyMetaMessages ) {}
ESaidTooManyMetaMessagesMsg::~ESaidTooManyMetaMessagesMsg() {}
ESaidTooManyAdviceMessagesMsg::ESaidTooManyAdviceMessagesMsg() 
  : ParsedMsg( E_SaidTooManyAdviceMessages ) {}
ESaidTooManyAdviceMessagesMsg::~ESaidTooManyAdviceMessagesMsg() {}
ESaidTooManyInfoMessagesMsg::ESaidTooManyInfoMessagesMsg() 
  : ParsedMsg( E_SaidTooManyInfoMessages ) {}
ESaidTooManyInfoMessagesMsg::~ESaidTooManyInfoMessagesMsg() {}
ESaidTooManyDefineMessagesMsg::ESaidTooManyDefineMessagesMsg() 
  : ParsedMsg( E_SaidTooManyDefineMessages ) {}
ESaidTooManyDefineMessagesMsg::~ESaidTooManyDefineMessagesMsg() {}
ESaidTooManyDelMessagesMsg::ESaidTooManyDelMessagesMsg() 
  : ParsedMsg( E_SaidTooManyDelMessages ) {}
ESaidTooManyDelMessagesMsg::~ESaidTooManyDelMessagesMsg() {}
ESaidTooManyRuleMessagesMsg::ESaidTooManyRuleMessagesMsg() 
  : ParsedMsg( E_SaidTooManyRuleMessages ) {}
ESaidTooManyRuleMessagesMsg::~ESaidTooManyRuleMessagesMsg() {}
ECouldNotParseSayMsg::ECouldNotParseSayMsg() : ParsedMsg( E_CouldNotParseSay ) {}
ECouldNotParseSayMsg::~ECouldNotParseSayMsg() {}
ESaidTooManyMessagesMsg::ESaidTooManyMessagesMsg() : ParsedMsg( E_SaidTooManyMessages ) {}
ESaidTooManyMessagesMsg::~ESaidTooManyMessagesMsg() {}

EUnknownErrorMsg::EUnknownErrorMsg( const char *msg ) : ParsedMsg( E_UnknownError ) {
  errorMsg = strdup( msg );
}
EUnknownErrorMsg::~EUnknownErrorMsg() {
  if ( errorMsg != NULL ) {
    free((void *)errorMsg);
  }
}

