/******************************************************************************
 * Parser.hh
 * Copyright (c) 2003 Virtual Werder
 *
 *
 * implements:
 *   class Parser
 *
 * requires:
 *
 *
 * Created on: 09.05.2003 by pagra
 * Maintainer: pagra
 * Group:
 * Co-authors: josh
 *****************************************************************************/

#ifndef PARSER_HH
#define PARSER_HH

#include "common.hh"
#include "ParsedObjects.hh"

/** @defgroup Parser Parser Classes
 *  This group contains all parser related classes
 *  @{
 */

class TeamGraphicOkMsg;
class ParsedMsg;
class SeeGlobalMsg;
class OkLookMsg;
class VisualMsg;
class OkTeamnamesMsg;
class CLangVersPlayerMsg;
class OkChangePlayerTypeMsg;
class ChangePlayerTypeMsg;
class PlayerTypeMsg;
class PlayerParamMsg;
class ServerParamMsg;

/**
 This class represents the parser, which takes messages from the coach (received
 by the communicator) and parses them. The main parsing method 'parseMsg' returns 
 a ParsedMsg, which holds all parseable information within the incoming message.
 @brief class Parser in group Parser
 */
class Parser {
public:
  Parser();
  ~Parser();
  void setTeamName (const char *teamname) { this->teamname = teamname; };
  ParsedMsg *parseMsg( char *msg );

private:
  const char *teamname;
  FieldSide ourSide;

  void swapSides(ParsedPlayer *p);

  TeamGraphicOkMsg *parseTeamGraphicOkMsg( char *msg ) const;
  ParsedMsg *parseHearMsg( char *msg ) const;
  SeeGlobalMsg *parseSeeGlobalMsg( char *msg ) const;
  OkLookMsg *parseOkLookMsg( char *msg ) const;
  void parseVisualData( char *msg, VisualMsg *parsedMsg ) const;
  OkTeamnamesMsg *parseOkTeamnamesMsg( char *msg ) const;
  CLangVersPlayerMsg *parseCLangVersPlayerMsg( char *msg ) const;
  OkChangePlayerTypeMsg *parseOkChangePlayerTypeMsg( char *msg ) const;
  ChangePlayerTypeMsg *parseChangePlayerTypeMsg( char *msg ) const;
  PlayerTypeMsg *parsePlayerTypeMsg( char *msg )const;
  PlayerParamMsg *parsePlayerParamMsg( char *msg ) const;
  ServerParamMsg *parseServerParamMsg( char *msg ) const;
};

/** @} */ // end of WorldModel

#endif // PARSER_HH
