/******************************************************************************
 * Player.cc
 * Copyright (c) 2003 Virtual Werder
 *
 *
 * implements:
 *   class Player
 *
 * requires:
 *
 *
 * Created on: 21.01.2004 by aw
 * Maintainer: aw, mangels
 * Group:
 * Co-authors:
 *
 * $Id: Player.cc,v 1.3 2004/06/24 09:30:39 aw Exp $
 *****************************************************************************/

#include "Player.hh"
#include "debug.hh"
#include "common.hh"


Player::Player(){
  goalie = false;
  type = new PlayerType();
  topSpeed = 0;
  accel = -1;
  decay = -1;
  decayNum = 0;
  kickableMargin = -1;
  for (int i = 0; i < 7; i++) {
    ptMatchingValues[i] = 0;
    //playerTypePossible[i] = true;
  }
  inertia = -1;
  playerType = -1;
  avgX = 0;
  avgY = 0;
  avgCycles = 0;
  typeChanged = false;
}


Player::Player(const int num){
  uNum = num;
  goalie = false;
  type = new PlayerType();
  topSpeed = 0;
  kickableMargin = -1;
  inertia = -1;
  playerType = -1;
  accel = -1;
  decay = -1;
  decayNum = 0;
  for (int i = 0; i < 7; i++) {
    ptMatchingValues[i] = 0;
    //playerTypePossible[i] = true;
  }
  avgX = 0;
  avgY = 0;
  avgCycles = 0;
  typeChanged = false;
  //Player();
}

Player::~Player(){
  
}


/*
 * Set Methods
 *
 */
void Player::setPosition(Vector2D pos){
  position = pos;
}


void Player::setPosition(const float posX, const float posY){
  position.setX(posX);
  position.setY(posY);
}

void Player::setVelocity(Vector2D vel){
  velocity = vel;
}

void Player::setVelocity(const float velX, const float velY){
  velocity.setX(velX);
  velocity.setY(velY);
}

void Player::setBodyDir(const float dir){
  bodyDir = dir;
}

void Player::setNeckDir(const float dir){
  neckDir = dir;
}


void Player::setType(PlayerType *t){
  type = t;
}

void Player::setUNum(const int num){
  uNum = num;
}

void Player::setGoalie(const bool isGoalie){
  goalie = isGoalie;
}

void Player::setTopSpeed(const float ts){
  topSpeed = ts;
}

void Player::setKickableMargin(const float km){
  this->kickableMargin = km;
}

void Player::setPredictedPlayerType (int i){
  this->predictedPlayerType = i;
}

void Player::setHasTypeChanged(bool changed) {
  this->typeChanged = changed;
}

void Player::updateAvgValues() {
   avgX += getPosX();
   avgY += getPosY();
   avgCycles++;
}


float Player::getPlayerTypeMatching(int ptId, float ptSpeedMax, float ptKickableMargin, float ptInertia, float ptAccel, float ptDecay){
  //if (!playerTypePossible[ptId]) return 0;
  bool debug = false;
#if defined (DEBUG_PLAYERTYPEMATCHING)
  debug = true;
#endif
  

  if (debug) printf (" Player %i (pt %i): ", uNum, ptId);

  float matchingValue = 0;
  int   speedFactor   = 6;
  int   marginFactor  = 1;
  int   inertiaFactor = 1;
  int   accelFactor   = 1;
  int   decayFactor   = 12;
  
  float diff = fabs(ptSpeedMax - this->topSpeed);
  
  if (debug) printf ("SpeedMax %.3f (%.2f%%) ", diff, (diff * 100) / ptSpeedMax);
  if (diff < ptSpeedMax * 0.05)
    matchingValue += (10 * speedFactor);
  else if (diff < ptSpeedMax * 0.10)
    matchingValue += ( 9 * speedFactor);
  else if (diff < ptSpeedMax * 0.17)
    matchingValue += ( 8 * speedFactor);
  else if (diff < ptSpeedMax * 0.27)
    matchingValue += ( 7 * speedFactor);
  else if (diff < ptSpeedMax * 0.40)
    matchingValue += ( 6 * speedFactor);
  else if (diff < ptSpeedMax * 0.59)
    matchingValue += ( 5 * speedFactor);
  else if (diff < ptSpeedMax * 0.85)
    matchingValue += ( 4 * speedFactor);
  
  
  
  /*
  if (diff < 0){
    playerTypePossible[ptId] = false;
    printf("   kicking type %i due to player speed, diff: %.4f\n", ptId, diff);
    return 0;
  }
  */
  
  diff = fabs(ptInertia - this->inertia);  
  if (debug) printf ("Inertia %.3f (%.2f%%) ", diff, (diff * 100) / ptInertia);
  if (this->inertia > 0){
    if (diff < ptInertia * 0.05)
      matchingValue += (10 * inertiaFactor);    
    else if (diff < ptInertia * 0.1)
      matchingValue += (9 * inertiaFactor);    
    else if (diff < ptInertia * 0.17)
      matchingValue += (8 * inertiaFactor);    
    else if (diff < ptInertia * 0.27)
      matchingValue += (7 * inertiaFactor);    
    else if (diff < ptInertia * 0.4)
      matchingValue += (6 * inertiaFactor);    
    else if (diff < ptInertia * 0.59)
      matchingValue += (5 * inertiaFactor);    
    else if (diff < ptInertia * 0.85)
      matchingValue += (4 * inertiaFactor);
  }

  /*
  if (this->inertia > 0 && diff > 0){
    playerTypePossible[ptId] = false;
    printf("   kicking type %i due to inertia, diff: %.4f\n", ptId, diff);
    return 0;
  }
  */


  if (this->accel > 0) {
    diff = fabs(ptAccel - this->accel);  
    if (debug) printf ("Accel %.3f (%.2f%%) ", diff, (diff * 100) / ptAccel);
    if (diff < ptAccel * 0.01)  // 1 %
      matchingValue += (10 * accelFactor);    
    else if (diff < ptAccel * 0.025) // 2.5 %
      matchingValue += (9 * accelFactor);    
    else if (diff < ptAccel * 0.06) // 6 %
      matchingValue += (8 * accelFactor);    
    else if (diff < ptAccel * 0.10) // 10 %
      matchingValue += (7 * accelFactor);    
    else if (diff < ptAccel * 0.17) // 17 %
      matchingValue += (6 * accelFactor);    
    else if (diff < ptAccel * 0.26) // 26 %
      matchingValue += (5 * accelFactor);    
  }


  /*
  if (this->accel > 0 && diff <= -1.0){
    playerTypePossible[ptId] = false;
    printf("   kicking type %i due to acceleration, diff: %.4f\n", ptId, diff);
    return 0;
  }
  */
  
  if (kickableMargin > 0) {		
    diff = fabs(ptKickableMargin - this->kickableMargin);
    if (debug) printf ("KickMargin %.3f (%.2f%%) ", diff, (diff * 100) / ptKickableMargin);
    if (diff < ptKickableMargin * 0.02) // 2%
      matchingValue += (10 * marginFactor);    
    else if (diff < ptKickableMargin * 0.04) // 4%
      matchingValue += (9 * marginFactor);    
    else if (diff < ptKickableMargin * 0.09) // 9%
      matchingValue += (8 * marginFactor);    
    else if (diff < ptKickableMargin * 0.15) // 15 %
      matchingValue += (7 * marginFactor);    
    else if (diff < ptKickableMargin * 0.23) // 23 %
      matchingValue += (6 * marginFactor);    
    else if (diff < ptKickableMargin * 0.33) // 33 %
      matchingValue += (5 * marginFactor);    
    else if (diff < ptKickableMargin * 0.47) // 47 %
      matchingValue += (4 * marginFactor);

    /*
    if (diff < -0.015){
      playerTypePossible[ptId] = false;
      printf("   kicking type %i due to kickable margin, margin: %.4f diff: %.4f\n", ptId, 
	     kickableMargin,
	     diff);
      return 0;
    }
    */
  }
  
  if (decay > 0) {		
    diff = fabs(ptDecay - this->decay);

    if (diff < ptDecay * 0.1){
      matchingValue += (100 * decayFactor * (ptDecay * 0.1 - diff));
    }
    if (debug) printf("Decay: %.3f (%.2f%%) ", diff, (diff * 100) / ptDecay);
  }

  if (debug) printf("Match: %.2f", matchingValue);
  if (debug) printf ("\n");
  
  //playerTypePossible[ptId] = matchingValue;
  ptMatchingValues[ptId] = matchingValue;

  return matchingValue;
}

bool Player::isFittestType(int ptId) {
  float value = 0;
  int type = 0;
  for (int j = 0; j < 7; j++) {
    if (ptMatchingValues[j] > value) {
      value = ptMatchingValues[j];
      type = j;
    }
  }
  if (ptId == type) return true;
  return false;
}

int Player::getFittestType() {
  float value = 0;
  int type = 0;
  for (int j = 0; j < 7; j++) {
    if (ptMatchingValues[j] > value) {
      value = ptMatchingValues[j];
      type = j;
    }
  }
  return type;
}

/*
bool Player::checkIfDoneWithPlayerTypes(){
  return false;
  
  if (playerType != -1) return true;
  int couldBe = -1;
  int count = 0;
  for (int i = 0; i < 7; i++){
    if (playerTypePossible[i]){
      couldBe = i;
      count++;
    }
  }
  if (count == 1){
    playerType = couldBe;
    return true;
  }
  if (count == 0){
    // reset
    for (int i = 0; i < 7; i++){
      playerTypePossible[i] = true;
      accel = -1;
      topSpeed = 0;
      inertia = -1;
      kickableMargin = -1;
    }
    return false;
  }
  return false;
}
*/
